"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = TemplateLiteralHandler;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var t = require('@babel/types');

function TemplateLiteralHandler(path, stylesCollector, stylesVarName) {
  if (path.node.expressions.length === 0) {
    // Simple template literal without any expressions
    var classNames = path.node.quasis.map(function (q) {
      return q.value.raw;
    }).join(' ');
    path.parentPath.replaceWith(t.StringLiteral(classNames));
  } else {
    // some expressions inside of the template literal
    var conditionals = path.node.expressions.filter(function (_ref) {
      var type = _ref.type,
          consequent = _ref.consequent,
          alternate = _ref.alternate;
      return type === 'ConditionalExpression' && ['TemplateLiteral', 'StringLiteral'].includes(consequent.type) && ['TemplateLiteral', 'StringLiteral'].includes(alternate.type);
    }).map(function (e) {
      e.consequent = t.ArrayExpression(getStringLiterals(e.consequent));
      e.alternate = t.ArrayExpression(getStringLiterals(e.alternate));
      stylesCollector.push.apply(stylesCollector, (0, _toConsumableArray2.default)(e.consequent.elements.map(function (s) {
        return s.value;
      })).concat((0, _toConsumableArray2.default)(e.alternate.elements.map(function (s) {
        return s.value;
      }))));
      return e;
    }); // Get all quasis and expressions sorted as they were in the original code

    var args = new Map(conditionals.map(function (c) {
      return [c.start, t.SpreadElement(c)];
    }));
    path.node.quasis.forEach(function (q) {
      var literals = getStringLiterals(q);
      stylesCollector.push.apply(stylesCollector, (0, _toConsumableArray2.default)(literals.map(function (l) {
        return l.value;
      })));
      args.set(q.start, literals);
    });
    var sortedArgs = Array.from(args.keys()).sort().map(function (i) {
      return args.get(i);
    }).flat();
    var jsxExpression = getJsxExpression(path, sortedArgs, stylesVarName);
    var jsxAttribute = t.jsxAttribute(t.jsxIdentifier('className'), t.jsxExpressionContainer(jsxExpression));
    path.parentPath.parentPath.replaceWith(jsxAttribute);
  }
}

function getJsxExpression(path, sortedArgs, stylesVarName) {
  var callExpressions = path.node.expressions.filter(function (e) {
    return e.type === 'CallExpression';
  });

  if (callExpressions.length === 0) {
    return t.callExpression(t.identifier(stylesVarName), sortedArgs);
  }

  var expressions = [t.callExpression(t.identifier(stylesVarName), sortedArgs)].concat((0, _toConsumableArray2.default)(callExpressions)).sort(function (a, b) {
    return a.start - b.start;
  });
  var quasis = expressions.map(function (e, i) {
    return t.templateElement({
      raw: i === 0 ? '' : ' '
    });
  });
  quasis.push(t.templateElement({
    raw: ''
  }, true));
  return t.templateLiteral(quasis, expressions);
}

function getStringLiterals(input) {
  if (!['StringLiteral', 'TemplateElement', 'TemplateLiteral'].includes(input.type)) {
    return [];
  }

  var strings = // eslint-disable-next-line no-nested-ternary
  input.type === 'StringLiteral' ? input.value.split(' ') : input.type === 'TemplateLiteral' ? input.quasis.map(function (q) {
    return q.value.raw.split(' ');
  }).flat() : input.value.raw.split(' ');
  return strings.filter(function (s) {
    return s !== '';
  }).map(function (s) {
    return t.StringLiteral(s);
  });
}